#pragma once
#include <string>
//...  header files,      .
#include "ppm.h"
#include "ImageStorageTraits.h"

namespace graphics
{
	class Image : public ImageStorageTraits
	{

	public:
		Image()
		{
		}

		Image(int w, int h)
		{
			if (w*h == 0)
				return;
			width = w;
			height = h;
			data = new unsigned char[3 * height * width];
		}

		unsigned char * operator () (unsigned int x, unsigned int y)
		{
			return pixel(x, y);
		}

		Image & operator = (const Image & orig)
		{
			if (&orig == this)
				return *this;
			if (data)
				delete[] data;
			width = orig.width;
			height = orig.height;
			data = new unsigned char[3 * height * width];
			memcpy(data, orig.data, 3 * height * width);
			return *this;
		}
		
		bool operator == (const Image & right)
		{
			if (width != right.width || height != right.height)
				return false;
			for (size_t i = 0; i < 3 * width*height; i++)
			{
				if (right.data[i] != data[i])
					return false;
			}
			return true;
		}

		Image(const Image & orig)
		{
			if (orig.height*orig.width == 0)
				return;
			width = orig.width;
			height = orig.height;
			data = new unsigned char[3 * height*width];
			memcpy(data, orig.data, 3 * height*width);
		}

		~Image()
		{
			if (data)
				delete[] data;
		}

		// ---------------- Test2 ---------------------------------------
		
		bool load(std::string filename)
		{
			if (data)
				delete[] data;
			width = height = 0;
			return (data = ReadPPM(filename, &width, &height));
		}

		std::pair<int, int> getDimensions()
		{
			return std::pair<int, int>(width, height);
		}

		bool save(std::string filename)
		{
			return WritePPM(data, width, height, filename);
		}
		
		// ---------------- Test3 ---------------------------------------
		
		operator unsigned char * ()
		{
			return data;
		}

		Image & operator << (const unsigned char * const buffer)
		{
			memcpy(data, buffer, width*height * 3);
			return *this;
		}

		Image & operator = (Image && orig)
		{
			if (&orig == this)
				return *this;
			if (data)
				delete[] data;

			width = orig.width;
			height = orig.height;

			if (height*width == 0)
				return *this;

			data = orig.data;
			orig.data = nullptr;
			orig.width = 0;
			orig.height = 0;

			return *this;
		}
		
	};
}
